extensions = ["sphinxcontrib.bibtex"]
exclude_patterns = ["_build"]
bibtex_bibfiles = ["test.bib"]
bibtex_cite_id = "cite-id-{bibliography_count}-{key}"
bibtex_footcite_id = "footcite-id-{key}"
bibtex_bibliography_id = "bibliography-id-{bibliography_count}"
bibtex_footbibliography_id = "footbibliography-id-{footbibliography_count}"
